-- backups inventory

local utils = require"core.pbxutils"
local fsutils = require"core.fsutils"

local backups = {
    path = require"config".backups.path,
    invlist = {},
    wms = "/rw2/var/lib/wmsdb.d/wmsdb"
}

-- copypasted from fileutils, directory fixed
local function readdir(dir_path, regexp, mode, first_match)
    local res_list = {}
    for file in require"lfs".dir(dir_path) do
        if file and "." ~= file and ".." ~= file then
            local check_mode = false
            local match_name = nil
            if regexp and regexp ~= "" then
                if "table" == type(regexp) then
                    for _, vregexp in pairs(regexp) do
                        match_name = file:match(vregexp)
                        if match_name then break end
                    end
                else
                    match_name = file:match(regexp)
                end
                if match_name then
                    check_mode = true
                end
            else
                check_mode = true
            end
            
            if check_mode then
                local obj_mode = nil
                if "file" == mode or "link" == mode then  
                    obj_mode = require"lfs".symlinkattributes(dir_path.."/"..file, "mode")
                elseif not mode or "directory" == mode then
                    obj_mode = require"lfs".attributes(dir_path.."/"..file, "mode")
                end
                    
                if mode == obj_mode or (not mode and ("file" == obj_mode or "link" == obj_mode)) then
                    if match_name then
                        table.insert(res_list, match_name)
                        if first_match then
                            break
                        end
                    else
                        table.insert(res_list, file)
                    end
                end
            end
        end
    end
    
    if #res_list > 0 then
        local sort_func = function(a,b) return a < b end
        table.sort(res_list, sort_func)
        return res_list
    end
    return nil, "Directory doesn't have files"
end

function backups.init(pbxses)
    local filter = nil
    if "table" == type(pbxses) then
        local acc = {}
        for _, id in pairs(pbxses) do
            acc[id] = true
        end
        filter = acc
    end

    if not fsutils.is_dir(backups.path) then
        return false, "backups folder doesn't exist"
    end

    local dirlist = readdir(backups.path, nil, "directory") or {}
    local counter = 0
    for _, dir in pairs(dirlist) do
        pbxpath = backups.path .. "/" .. dir
        archfiles = readdir(pbxpath) or {}
        archtable = {}
        local hostname = nil
        for _, archive in pairs(archfiles) do
            local host, y, m, d, h, min, sec = string.match(archive, "([%d%l%-]+)_(%d%d%d%d)%.(%d%d)%.(%d%d)_(%d%d)%.(%d%d)%.(%d%d)_")
            if host then
                hostname = string.gsub(host, "%-", "")
                local timetable = {
                    ["year"] = y,
                    ["month"] = m,
                    ["day"] = d,
                    ["hour"] = h,
                    ["min"] = min,
                    ["sec"] = sec
                }
                local timestamp = os.time(timetable)
                archtable[#archtable + 1] = {timestamp, archive}
            end
        end
        if hostname then
            if not filter or filter[hostname] then
                table.sort(archtable, function(a,b) return a[1] > b[1] end)
                local lastarch = archtable[1]
                backups.invlist[hostname] = {["timestamp"] = lastarch[1], ["archive"] = pbxpath .. "/" .. lastarch[2]}
                counter = counter + 1
            end
        end
    end

    if counter == 0 then
        return false, "no pbxes found"
    end
    return true
end

function backups.get_pbxses()
    local res = {}
    for id, _ in pairs(backups.invlist) do
        res[#res + 1] = id
    end
    return res
end

function backups.get_pbx_meta(host)
    return backups.invlist[host]
end

function backups.prepare(host)
    local data = backups.invlist[host]
    if not data then
        return false, "pbx not found in repository"
    end
    local archive = data["archive"]
    local file = require"posix".basename(archive)
    data["localfile"] = "/tmp/" .. file
    local rv = fsutils.cpfile(archive, "/tmp")
    if not rv then
        return false, "copy backup archive error"
    end

    -- TODO error check
    local command1 = "tar -C /tmp -zxf %s wmsdb.sql"
    utils.run(command1, data["localfile"])

    local command2 = "sqlite3 /tmp/wmsdb < /tmp/wmsdb.sql"
    utils.run(command2)

    fsutils.cpfile(backups.wms, "/tmp/wmsdb.bak")
    fsutils.cpfile("/tmp/wmsdb", backups.wms)
    return true
end

function backups.cleanup(host)
    if fsutils.is_file("/tmp/wmsdb.bak") then
        fsutils.cpfile("/tmp/wmsdb.bak", backups.wms)
    end

    os.remove("/tmp/wmsdb")
    os.remove("/tmp/wmsdb.bak")
    os.remove("/tmp/wmsdb.sql")

    local archive = backups.invlist[host]["localfile"]
    if archive then
        os.remove(archive)
    end
end

return backups
