-- module for retain and update db ranges
-- require file with empty table {} on start

local conf = require"config"

local idrange = {
    filename = conf.idrange and conf.idrange.path or "/var/lib/callweaver/dialplan/failover/configuration_merger/libs/idranges.json",
    step = conf.idrange and conf.idrange.step or 1000000,
    idtable = nil
}

local function read_table()
    local fp = io.open(idrange.filename)
    if fp then
        idrange.idtable = require"dkjson".decode(fp:read("*a"))
        io.close (fp)
    end
    return idrange.idtable
end

local function get_next_number()
    local max = 0
    for _, val in pairs(idrange.idtable) do
        if val > max then
            max = val
        end
    end
    return max + idrange.step
end

local function update()
    local fp = io.open(idrange.filename, "w")
    fp:write(require"dkjson".encode(idrange.idtable))
    fp:close()
end

-- id is hostname, returns first in range and numbers in range
function idrange.get_range(id)
    if not idrange.idtable and not read_table() then
        return nil, "error reading range json file"
    end
    local start = idrange.idtable[id]
    if start then
        return start, start + idrange.step - 1
    else
        local newstart = get_next_number()
        idrange.idtable[id] = newstart
        update()
        return newstart, newstart + idrange.step - 1
    end
end

return idrange
