#!/usr/bin/lua

-- update prepared data in wmsdb
-- use stdout for errors messages
-- TODO all in one transaction?

local input = arg[1]
local data = nil
local fp = io.open(input)
if fp then
	data = require"dkjson".decode(fp:read ("*a"))
	io.close(fp)
end

if not data then
	print("input file error")
	os.exit(1)
end

local range_start_above = data["range"][1]
local range_stop_below = data["range"][2]

local function handle_error(err, msg)
	if err == "Database error" then
		print("error in " .. msg)
		os.exit(1)
	end
end

local utils = require"core.pbxutils"
local connection = require"common.core".connection
connection:switch ("wmsdb")
local sql = nil

local tables = {"timetable_main", "wmsswitches", "call_groups_members", "call_groups", "timetable_additional"}

-- delete with ranges
sql = "delete from %s where id between %d and %d"
for _, t in pairs(tables) do
	local r, err = connection:execute(sql, t, range_start_above, range_stop_below)
	handle_error(err, "delete from " .. t)
end

-- inserts
local timetable_main = data["timetable_main"]
sql = "insert into timetable_main (id, state, timetable, desc) values (%d, %d,'%s', '%s')"
for _, vals in pairs(timetable_main) do
	local r, err = connection:execute(sql, vals["id"], vals["state"], vals["timetable"], vals["desc"])
	handle_error(err, "insert into timetable_main")

	local state, setstate = tonumber(vals["state"]), "RINGING"
    if state == 0 then
        setstate = "NOT_INUSE"
    elseif state == 1 then
        setstate = "INUSE"
    elseif state == 2 then
        setstate = "RINGING"
    end
	utils.run([[/usr/sbin/callweaver -rx "devstate change Custom:DS/stime*%s*1 %s"]],  vals["id"], setstate)
end

local wmsswitches = data["wmsswitches"]
sql = "insert into wmsswitches (id, state, switchname, swtype) values (%d, %d,'%s', %d)"
for _, vals in pairs(wmsswitches) do
	local r, err = connection:execute(sql, vals["id"], vals["state"], vals["switchname"], vals["swtype"])
	handle_error(err, "insert into wmsswitches")

	local state, setstate = tonumber(vals["state"]), "RINGING"
    if state == 0 then
        setstate = "NOT_INUSE"
    elseif state == 1 then
        setstate = "INUSE"
    elseif state == 2 then
        setstate = "RINGING"
    end
	utils.run([[/usr/sbin/callweaver -rx "devstate change Custom:DS/%s*%s*1 %s"]], tonumber(vals["swtype"]) == 3 and "3switch" or "switch", vals["id"], setstate)
end

local call_groups_members = data["call_groups_members"]
sql = "insert into call_groups_members (id, call_group_id, group_member, group_membername) values (%d, %d,'%s', '%s')"
for _, vals in pairs(call_groups_members) do
	local r, err = connection:execute(sql, vals["id"], vals["call_group_id"], vals["group_member"], vals["group_membername"])
	handle_error(err, "insert into call_groups_members")
end


local call_groups = data["call_groups"]
sql = "insert into call_groups (id, queue_parameters, call_groupname) values (%d, '%s', '%s')"
for _, vals in pairs(call_groups) do
	local r, err = connection:execute(sql, vals["id"], vals["queue_parameters"], vals["call_groupname"])
	handle_error(err, "insert into call_groups")
end

local timetable_additional = data["timetable_additional"]
sql = "insert into timetable_additional (id, parent_id, day_id_from, day_id_to, day_of_week_from, day_of_week_to, year_id, enabled_id, month_id, start_daytime, end_daytime) values (%d, %d, %d, %d, %d, %d, %d, %d, %d, '%s', '%s')"
for _, vals in pairs(timetable_additional) do
	local r, err = connection:execute(sql, vals["id"], tonumber(vals["parent_id"]), vals["day_id_from"], vals["day_id_to"], vals["day_of_week_from"], vals["day_of_week_to"], vals["year_id"], vals["enabled_id"], vals["month_id"], vals["start_daytime"], vals["end_daytime"])
	handle_error(err, "insert into timetable_additional")
end

connection:close()
print("completed")
