local sounds = {}

local fsutils = require"core.fsutils"
local pbxutils = require"core.pbxutils"

function sounds.prepare(host, task)
	if not host or "table" ~= type(task) then
		return false, "pbx does not found while preparing data"
	end
	local archive = task["archive"]
	if not archive or "" == archive then
		return false, "pbx name does not exist"
	end
	local untar = "/bin/tar -C /tmp -xzvf %s cust_sounds.tar"
	pbxutils.run(untar, archive)
	local target_dir = string.format("/tmp/%s", host)
	fsutils.mkdir(target_dir)
	local untar = "/bin/tar -xvf %s -C %s"
	pbxutils.run(untar, "/tmp/cust_sounds.tar", target_dir)
	-- TODO check that untar is correct and dir exists

	return true
end

function sounds.enroll(host, task)
	if not host or "table" ~= type(task) then
		return false, "pbx does not found while enrolling data"
	end
    -- TODO use rsync ansible module?
    local utils = require"core.pbxutils"
    local playbook = "/var/lib/callweaver/dialplan/failover/configuration_merger/playbooks/sounds.yml"
    local extra = "/var/lib/callweaver/dialplan/failover/configuration_merger/playbooks/extra.json"
    local remote_host = task["config"]["dstpbx"]["apihost"]

    local command = 'ansible-playbook -i %s, %s --extra-vars "@%s" --extra-vars "host_name=%s"'
    local out = utils.run(command, remote_host, playbook, extra, host)

    if string.find(out, "unreachable=0") and string.find(out, "failed=0") then
        return true
    end

    return false, string.gsub(out, "[\r\n]", "")

end

function sounds.cleanup(host)
	if not host then
		return false, "pbx does not found while cleaning up data"
	end
	if not fsutils.is_dir("/tmp/" .. host) then
       return false, "sound files does not found while cleaning up data"
    end
    fsutils.rmdirr("/tmp/" .. host)
    return true
end

return sounds

