local wmsdb_sync = {}


function wmsdb_sync.prepare(host, task)
    if not host or host == "" then
        return false, "empty host"
    end

    local prefix, rangeend = require"libs.idrange".get_range(host)
    if not prefix then
        return false, rangeend
    end

    local utils = require"core.pbxutils"
    local connection = require"common.core".connection

    local tables = {"call_groups", "call_groups_members", "wmsswitches", "timetable_main", "timetable_additional"}

    local function fix_id(tablename, rec, host)
        local to_fix = {call_groups = {"id"},
                        call_groups_members = {"id", "call_group_id"},
                        wmsswitches = {"id"},
                        timetable_main = {"id"},
                        timetable_additional = {"id", "parent_id"}
                        }
        local t = {}
        for k, v in pairs(rec) do
            if utils.in_table(k, to_fix[tablename]) then
                t[k] = prefix + tonumber(v)
            elseif (tablename == "timetable_main" and k == "timetable") or
                   (tablename == "call_groups" and k == "call_groupname") or
                   (tablename == "wmsswitches" and k == "switchname") then
                t[k] = string.sub(string.format("%s_%s", host, v), 1, 38) .. string.sub(tostring(rec["id"]), -2)
            else
                t[k] = v
            end

        end
        return t
    end
    
    local rv = {}
    rv.range = {prefix, rangeend}
    for _, tablename in ipairs(tables) do rv[tablename] = {} end

    connection:switch("wmsdb")
    for _, tablename in ipairs(tables) do
        local sql = "SELECT * FROM `%s`;"
        local cursor, err = connection:execute(sql, tablename)
        if not cursor then
            return false, "sql query failed, problems with db are possible"
        end

        local row = cursor:fetch({}, "a")
        while row do
            table.insert(rv[tablename], fix_id(tablename, row, host))
            row = cursor:fetch({}, "a")
        end
        cursor:close()
    end
    connection:close()

    local json = require"dkjson".encode(rv)
    if not json or json == "null" then
        return false, "json encode error"
    end

    local fp = io.open("/tmp/sqlite_data.json", "w")
    fp:write(json)
    fp:close()

    task["modules"]["wmsdb_sync"] = rv
    return true
end


function wmsdb_sync.enroll(host, task)
    local utils = require"core.pbxutils"
    local playbook = "/var/lib/callweaver/dialplan/failover/configuration_merger/playbooks/wmsdb.yml"
    local extra = "/var/lib/callweaver/dialplan/failover/configuration_merger/playbooks/extra.json"
    local remote_host = task["config"]["dstpbx"]["apihost"]
    local command = 'ansible-playbook -i %s, %s --extra-vars "@%s"'
    local out = utils.run(command, remote_host, playbook, extra)

    if string.find(out, "unreachable=0") and string.find(out, "failed=0") and string.find(out, "completed") then
        return true
    end

    return false, string.gsub(out, "[\r\n]", "")
end


function wmsdb_sync.cleanup(host, task)
    os.remove("/tmp/sqlite_data.json")
    return true
end

return wmsdb_sync

