#!/usr/bin/lua5.1

package.path = package.path .. ";/var/lib/callweaver/dialplan/failover/?.lua"

local h = require"common.helper"
local os = require"os"
local pbxutils = require"core.pbxutils"

print("Loading trunk parameters from trunk_params.lua ...")
dofile("/var/lib/callweaver/dialplan/failover/trunk_settings.lua")
print("Loading pbx list from pbx.lua .. ")
local gateways = require"pbx"


function parse_gateway(gw)
    local host, ports_num = string.match(gw, "^(.*):(%d+)$")
    if not host or not ports_num or not pbxutils.get_ipv4(host) then
        print("Invalid gw string: " .. tostring(gw))
        os.exit(1)
    end
    return host, ports_num
end


local connection = require"common.core".connection
connection:switch("wmsdb")

print("Trying to delete previously created trunks..")
local regexp = "____________e_%e__"
_, err = connection:execute("DELETE FROM trunks WHERE username LIKE '%s' ESCAPE 'e';", regexp)
if not err then
    print("Deleted\r\n")
else
    print(err)
    os.exit(1)
end


local cursor = connection:execute("SELECT MAX(id) FROM trunks;")
local wms_id = tonumber(cursor:fetch()); cursor:close()
wms_id = wms_id and wms_id + 1 or 1
for pbx in pairs(gateways) do
    print("Processing pbx " .. pbx .. ":\r\n")
    for gateway_id in pairs(gateways[pbx]["pri_bri"]) do
        local host, ports_num = parse_gateway(gateways[pbx]["pri_bri"][gateway_id])
        for port = 1, ports_num do
            local username = string.format("%s_%s_%s", pbx, gateway_id, port)
            settings_pbx["peername"] = username
            settings_pbx["username"] = username
            settings_pbx["host"] = host
            settings_pbx["port"] = ports[port]
            local hostname = pbxutils.split(gateways[pbx]["domain"], "%.")[1]
            if not hostname or hostname == "" then
                settings_pbx["context"] = "main"
                print(string.format("WARNING! Can't get a hostname from the PBX domain name. '%s' will be used instead. PBX: '%s'", settings_pbx["context"], pbx))
            else
                settings_pbx["context"] = string.gsub(hostname, "%-", "") .. "_main"
            end

            connection:switch("wmsdb")
            local sql = "INSERT INTO trunks (id,username,title,register,settings_pbx,settings_wms) VALUES (%d,%q,%q,'','%s','%s')"
            connection:execute(sql, wms_id, username, username, pbxutils.tojson(settings_pbx), pbxutils.tojson(settings_wms))
            
            connection:switch("htables")
            local sql = "INSERT INTO %s (key_name,key_value,wms_id) VALUES (%q,%q,%q)"
            connection:execute(sql, "static_peers", username .. "::host", settings_pbx["host"], "10" .. wms_id)
            connection:execute(sql, "static_peers", username .. "::authname", username, "10" .. wms_id)
            connection:execute(sql, "static_peers", username .. "::use_rport", settings_pbx["rport"], "10" .. wms_id)

            wms_id = wms_id + 1
            print(string.format("Added trunk %s ", username))
        end
    end
    print(string.rep("-", 30))
end

connection:close()

print("Generating sip-extra.conf...")
h.create_sip_extra()
print("Reloading htables...")
os.execute "sudo kamcmd htable.reload static_peers"
print("Reloading callweaver...")
os.execute "sudo callweaver -rx \"sip reload \" > /dev/null"

print("Done!")
